{------------------------------------------------------------------}
{ Component:	TbspRunningText										}
{ Purpose:		Displays a running text in a label					}
{ Properties:														}
{	Active		starts/stops the text (at least Continuos or		}
{										NumRepeat must be set		}
{	Continuous	lets the text run and run and run and run...		}
{	Direction	should be selfexplaining							}
{	NumRepeat	  "		"		"									}
{	Speed		the smaller the faster (depends from Steps			}
{	Steps		the smaller the smoother (depends from speed		}
{	TextLayout	where the text is positioned in the label			}
{	TextStyle	normal, lowered or raised text						}
{	RunningCaption the most importand property						}
{																	}
{ In the design environment there is a border around the label		}
{ In runtime modus this border will not be painted					}
{																	}
{ Known problems:													}
{	Changing the Transparent has no effect if Active=True			}
{																	}
{ Licencing,...													}
{	This component is absolutely freeware							}
{	But for that reason you also use it at your own risk!			}
{																	}
{  1999 by bsp ag, basel, switzerland								}
{	written by Remy Schnell											}
{	eMail: rschnell@ bsp-brainware.com								}
{------------------------------------------------------------------}
unit bspRunningText;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  stdCtrls, extctrls;

type
  TbspRunDirection = (diLeftToRight, diRightToLeft, diTopToBottom, diBottomToTop);
  TbspRunFontStyle = (slNormal, slLowered, slRaised);
  TbspRunTextLayout = (tlTop, tlCenter, tlBottom);

  TbspRunningText = class(TCustomlabel)
  private
	 { Private-Deklarationen }
	 FBitmap: TBitmap;
	 FTimer: TTimer;
	 FCurCycle: Integer;
	 FActive: Boolean;
	 FDirection: TbspRunDirection;
	 FStyleLabel: TbspRunFontStyle;
	 FLayout: TbspRunTextLayout;
	 FSpeed: Integer;
	 FNumRepeat: Word; 			//number of repeat cycles
	 FContinuous: Boolean;		// same that FNumRepeat = infinity
	 FSteps: Integer;           // number of steps
	 CurrentStep: Integer;      // current step when running
	 FRunCaption: string;		// text to show running
	 procedure SetActive(Value: Boolean);
	 procedure SetDirection(Value: TbspRunDirection);
	 procedure SetStyle(Value: TbspRunFontStyle);
	 procedure SetSteps(Value: Integer);
	 procedure SetSpeed(Value: Integer);
	 procedure SetLayout(Value: TbspRunTextLayout);
  protected
	 { Protected-Deklarationen }
	 procedure TimerExpired(Sender: TObject);
	 procedure DrawRunText;
	 // hide following properties
	 property Autosize;
	 property WordWrap;
	 property Caption;
  public
	 { Public-Deklarationen }
	 constructor Create(AOwner: TComponent); override;
	 destructor Destroy; override;
	 procedure Paint; override;
  published
	 { Published-Deklarationen }
	 property Active: Boolean read FActive write SetActive;
	 property Direction: TbspRunDirection read FDirection write SetDirection;
	 property TextStyle: TbspRunFontStyle read FStyleLabel write SetStyle;
	 property NumRepeat: Word read FNumRepeat write FNumRepeat;
	 property Continuous: Boolean read FContinuous write FContinuous;
	 property Steps: Integer read FSteps write SetSteps;
	 property Speed: Integer read FSpeed write SetSpeed;
	 property TextLayout: TbspRunTextLayout read FLayout write SetLayout default tlTop;
	 property RunningCaption: string read FRunCaption write FRunCaption;
	 // publish properties
	 property Align;
	 property Alignment;
	 property Anchors;
	 property BiDiMode;
	 property Color;
	 property Constraints;
	 property DragCursor;
	 property DragKind;
	 property DragMode;
	 property Enabled;
	 property FocusControl;
	 property Font;
	 property ParentBiDiMode;
	 property ParentColor;
	 property ParentFont;
	 property ParentShowHint;
	 property PopupMenu;
	 property ShowHint;
	 property Transparent;
	 property Visible;
	 property OnClick;
	 property OnDblClick;
	 property OnDragDrop;
	 property OnDragOver;
	 property OnEndDock;
	 property OnEndDrag;
	 property OnMouseDown;
	 property OnMouseMove;
	 property OnMouseUp;
	 property OnStartDock;
	 property OnStartDrag;
  end;

  procedure Register;

implementation
{$R bsp_runtxt.dcr}

procedure TbspRunningText.Paint;
begin
	inherited paint;
	if (csDesigning in ComponentState) then
	begin
		Canvas.Brush.Color := clBlack;
		Canvas.FrameRect(Canvas.ClipRect);
	end;
end;

// creates the timer an sets the defaults
constructor TbspRunningText.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  // set values of protected properties
  Autosize := False;
  Wordwrap := False;
  // set default size
  Width := 180;
  Height := 40;
  // patch to hide the original caption
  Name := 'bspRunningText';
  Caption := '';
  // set default property values
  FSteps := 100;
  CurrentStep := 0;
  FSpeed := 100;
  // create the bitmap holding the background image
  FBitmap := TBitmap.Create();
  // create the timer
  FTimer := TTimer.Create(Self);
  with FTimer do
  begin
	 Enabled := False;
	 OnTimer := TimerExpired;
	 Interval := FSpeed;
  end;
end;

// frees the timer / background bitmap
destructor TbspRunningText.Destroy;
begin
  FTimer.Free;
  FBitmap.free;
  inherited Destroy;
end;

// sets active property
procedure TbspRunningText.SetActive(Value: Boolean);
var
	lCaption:		string;
begin
	if (FActive <> Value) then
	begin
		// save the actual background into FBitmap (needed in DrawRunText procedure)
		if (Value = True) then
		begin
			With FBitmap do
			begin
				Width := self.Width;
				Height := self.Height;
				Canvas.CopyRect(self.Canvas.ClipRect, self.Canvas, self.Canvas.ClipRect);
			end;
		end;
		FActive := Value;
		FCurCycle := 0;
		CurrentStep := 0;
		FTimer.Enabled := Value;
		if (Value = False) then
		begin
			Invalidate;
			DrawRunText;
		end;
	end;
end;

// ontimer handling
procedure TbspRunningText.TimerExpired(Sender: TObject);
begin
  if not FTimer.Enabled then Exit;
  if (FCurCycle < FNumRepeat) or FContinuous then
  begin
	 Inc(CurrentStep);
	 DrawRunText;			// draw the running text
	 if CurrentStep = FSteps then
	 begin
	   CurrentStep := 0;
	   if not FContinuous then
		 Inc(FCurCycle);
	 end;
  end
  else
	 Active := False;
end;

// set direction property
procedure TbspRunningText.SetDirection;
begin
  if (FDirection <> Value) then
  begin
	 FDirection := Value;
	 if FActive then
	   Invalidate;
  end;
end;

// set label style property
procedure TbspRunningText.SetStyle(Value: TbspRunFontStyle);
begin
  if FStyleLabel <> Value then
  begin
	 FStyleLabel := Value;
	 Invalidate;
  end;
end;

// set step property
procedure TbspRunningText.SetSteps(Value: Integer);
begin
  if FSteps <> Value then
  begin
	 FSteps := Value;
	 if (csDesigning in ComponentState) then
	 begin
	   Invalidate;
	 end;
  end;
end;

// set speed property
procedure TbspRunningText.SetSpeed(Value: Integer);
begin
  if FSpeed <> Value then
  begin
	 FSpeed := Value;
	 if Value > 1000 then FSpeed := 1000;
	 if Value < 1 then FSpeed := 1;
	 // Change the timer interval
	 if FTimer <> nil then
	   FTimer.Interval := FSpeed;
  end;
end;

// set layout
procedure TbspRunningText.SetLayout(Value: TbspRunTextLayout);
begin
  if FLayout <> Value then
  begin
	 FLayout := Value;
	 Invalidate;
  end;
end;

// paint the whole stuff
procedure TbspRunningText.DrawRunText;
const
	Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
	WordWraps: array[Boolean] of Word = (0, DT_WORDBREAK);
	Layouts: array[TTextLayout] of Word = (DT_TOP, DT_VCENTER, DT_BOTTOM);
var
	Rect, CalcRect: TRect;
	StX, StY: Integer;
	intWidth, intHeight: Integer;
	CnX, CnY: Integer;
	PctDone: Double;
begin
	// refresh the bitmap
	Paint;
	// paint the text
	with Canvas do
	begin
		 // Calculate start points
		intWidth := TextWidth(RunningCaption) + 2;
		intHeight := TextHeight(RunningCaption) + 2;
		case Alignment of
			taLeftJustify:
				CnX := 0;
			taRightJustify:
				CnX := Width - intWidth;
			taCenter:
				CnX := (Width - intWidth) div 2;
		end;
		case TextLayout of
			tlTop:
				CnY := 0;
			tlBottom:
				CnY := Height - intHeight;
			tlCenter:
				CnY := (Height - intHeight) div 2;
		end;
		//Calculate percentages & starting points
		PctDone := CurrentStep/FSteps;
		if Active then
		begin
			case FDirection of
				diRightToLeft:
				begin
					StY := CnY;
					StX := -intWidth + Round((intWidth + Width)*(1 - PctDone));
				end;
				diLeftToRight:
				begin
					StY := CnY;
					StX := -intWidth + Round((intWidth + Width)*(PctDone));
				end;
				diTopToBottom:
				begin
					StX := CnX;
					StY := -intHeight + Round((intHeight + Height)*(PctDone));
				end;
				diBottomToTop:
				begin
					StX := CnX;
					StY := -intHeight + Round((intHeight + Height)*(1 - PctDone));
				end;
			end;
		end
		else
		begin // static label
			StX := CnX;
			StY := CnY;
		end;
		// reset the background
		CopyRect(ClipRect, FBitmap.Canvas, ClipRect);
		// draw the text
		Font := Self.Font;
		Brush.Style := bsClear;
		case FStyleLabel of
			slRaised:
			begin
				Font.Color := clBtnShadow;
				TextOut(StX + 1, StY + 1, RunningCaption);
			end;
			slLowered:
			begin
				Font.Color := clBtnHighlight;
				TextOut(StX + 1, StY + 1, RunningCaption);
			end;
		end;
		Font.Color := Self.Font.Color;
		TextOut(StX, StY, RunningCaption);
	end;

end;

// register the component
procedure Register;
begin
  RegisterComponents('bsp', [TbspRunningText]);
end;

end.
